#include <stdlib.h>
#include <xdc/runtime/System.h>
#include <xdc/std.h>

/* Drivers */
#include <ti/drivers/rf/RF.h>
#include <ti/drivers/PIN.h>

/* Application headers */
#include "Board.h"
#include "menu.h"
#include "RFQueue.h"
#include "smartrf_settings/smartrf_settings.h"


/***** Defines *****/
#define PAYLOAD_LENGTH          1000//250//30
#define DATA_ENTRY_HEADER_SIZE  8    /* Constant header size of a Generic Data Entry */
#define MAX_LENGTH              1000//125  /* Set the length of the data entry */
#define NUM_DATA_ENTRIES        1  
#define NUM_APPENDED_BYTES      0


/***** Variable declarations *****/
static RF_Object rfObject;
static RF_Handle rfHandle;

static uint8_t packet[PAYLOAD_LENGTH];
static uint16_t seqNumber;
uint32_t packetCounter = 0;
uint32_t packetInterval;

#if defined(__TI_COMPILER_VERSION__)
#pragma DATA_ALIGN (txDataEntryBuffer, 4);
#elif defined(__IAR_SYSTEMS_ICC__)
#pragma data_alignment = 4
#else
#error This compiler is not supported.
#endif
static uint8_t txDataEntryBuffer[RF_QUEUE_DATA_ENTRY_BUFFER_SIZE(NUM_DATA_ENTRIES,
                                                                 MAX_LENGTH,
                                                                 NUM_APPENDED_BYTES)];
/* Tx queue or RF Core to read data from */
static dataQueue_t dataQueue;
static rfc_dataEntryGeneral_t* currentDataEntry;
static uint8_t *pPacket;

#define WOR_PREAMBLE_TIME_RAT_TICKS(x) ((uint32_t)(4000000*(1.0f/(x)))) 

TestCaseResult tx_run()
{  
  uint32_t time;
  
  RF_Params rfParams;
  RF_Params_init(&rfParams);
  
  RF_cmdPropTxAdv.pktLen = PAYLOAD_LENGTH; /* +1 for length byte */
  RF_cmdPropTxAdv.pPkt = packet;
  RF_cmdPropTxAdv.startTrigger.triggerType = TRIG_ABSTIME;
  RF_cmdPropTxAdv.startTrigger.pastTrig = 1;
  RF_cmdPropTxAdv.startTime = 0;
//  RF_cmdPropTxAdv.preTime = WOR_PREAMBLE_TIME_RAT_TICKS(100);
  if( RFQueue_defineQueue(&dataQueue,
                          txDataEntryBuffer,
                          sizeof(txDataEntryBuffer),
                          NUM_DATA_ENTRIES,
                          MAX_LENGTH + NUM_APPENDED_BYTES))
  {
    /* Failed to allocate space for all data entries */
    while(true);
  }
  currentDataEntry = (rfc_dataEntryGeneral_t*)&txDataEntryBuffer;
  currentDataEntry->length = PAYLOAD_LENGTH; /* Length of the packet to send */
  pPacket = &currentDataEntry->data;
  
  switch (menu_config.testCase)
  {
  case FskTest:
    if(ppFrequencies[menu_config.frequency].frequency == 0x01b1)
    {
      RF_cmdPropRadioDivSetup_fsk.txPower = 0x003f;
      RF_cmdPropRadioDivSetup_fsk.loDivider = 0x0a;
    }  
    RF_cmdPropRadioDivSetup_fsk.centerFreq = ppFrequencies[menu_config.frequency].frequency;
    rfHandle = RF_open(&rfObject, &RF_prop_fsk, (RF_RadioSetup*)&RF_cmdPropRadioDivSetup_fsk, &rfParams);
    packetInterval = (uint32_t)(4000000*0.5f); /* Set packet interval to 20 ms */
    
    break;
    
  case OneMTest:
    if(ppFrequencies[menu_config.frequency].frequency == 0x01b1)
    {
      RF_cmdPropRadioDivSetup_1M.txPower = 0x003f;
      RF_cmdPropRadioDivSetup_1M.loDivider = 0x0a;
    }
    RF_cmdPropRadioDivSetup_1M.centerFreq = ppFrequencies[menu_config.frequency].frequency;  
    rfHandle = RF_open(&rfObject, &RF_prop_1M, (RF_RadioSetup*)&RF_cmdPropRadioDivSetup_1M, &rfParams);
    packetInterval = (uint32_t)(4000000*0.5f); /* Set packet interval to 10 ms */
    break;
    
  case Test500K:
    if(ppFrequencies[menu_config.frequency].frequency == 0x01b1)
    {
      RF_cmdPropRadioDivSetup_500K.txPower = 0x003f;
      RF_cmdPropRadioDivSetup_500K.loDivider = 0x0a;
    }
    RF_cmdPropRadioDivSetup_500K.centerFreq = ppFrequencies[menu_config.frequency].frequency;  
    rfHandle = RF_open(&rfObject, &RF_prop_500K, (RF_RadioSetup*)&RF_cmdPropRadioDivSetup_500K, &rfParams);
    packetInterval = (uint32_t)(4000000*0.5f); /* Set packet interval to 20 ms */
    break;
    
  default:
    
    break;
  }

  RF_cmdFs.frequency = ppFrequencies[menu_config.frequency].frequency;
  RF_cmdFs.fractFreq = ppFrequencies[menu_config.frequency].fractFreq;
  
  RF_postCmd(rfHandle, (RF_Op*)&RF_cmdFs, RF_PriorityNormal, NULL, 0);

  time = RF_getCurrentTime();
  while (++packetCounter <= menu_config.packetCount)
  {
    pPacket[0] = packet[0] = (uint8_t)(seqNumber >> 8);
    pPacket[1] = packet[1] = (uint8_t)(seqNumber++);
    uint16_t i;
    for (i = 2; i < PAYLOAD_LENGTH; i++)
    {
      pPacket[i] = packet[i] = rand();
    }
    
    time += packetInterval;
    RF_cmdPropTxAdv.startTime = time;
    
    RF_EventMask result = RF_runCmd(rfHandle, (RF_Op*)&RF_cmdPropTxAdv, RF_PriorityNormal, NULL, 0);
    if (result != RF_EventLastCmdDone)
    {
      while(true);
    }
    menu_update_txScreen(packetCounter);
    
    if (Event_pend(menu_event, Event_Id_NONE, KeyPressEvent, BIOS_NO_WAIT) & KeyPressEvent)
    {
      packetCounter = 0;
      seqNumber = 0;
      RF_close(rfHandle);
      return TestAborted;
    }
  }
  
  packetCounter = 0;
  seqNumber = 0;
  RF_close(rfHandle);
  return TestFinished;
}